// -*- C++ -*-
/*!
 * @file RingBuffer.h
 * @brief Defautl Buffer class
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RingBuffer.h 1963 2010-04-22 08:41:25Z n-ando $
 *
 */

#ifndef RTC_RINGBUFFER_H
#define RTC_RINGBUFFER_H

#include <vector>
#include <algorithm>
#include <iostream>

#include <coil/TimeValue.h>
#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <coil/Condition.h>
#include <coil/stringutil.h>

#include <rtm/BufferBase.h>
#include <rtm/BufferStatus.h>

#define RINGBUFFER_DEFAULT_LENGTH 8
/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RT$B%3%s%]!<%M%s%H(B
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class RingBuffer
   * @brief $B%j%s%0%P%C%U%!<BAu%/%i%9(B
   * 
   * $B;XDj$7$?D9$5$N%j%s%0>u%P%C%U%!$r;}$D%P%C%U%!<BAu%/%i%9!#(B
   * $B%P%C%U%!A4BN$K%G!<%?$,3JG<$5$l$?>l9g!"0J9_$N%G!<%?$O8E$$%G!<%?$+$i(B
   * $B=g<!>e=q$-$5$l$k!#(B
   * $B=>$C$F!"%P%C%U%!Fb$K$OD>6a$N%P%C%U%!D9J,$N%G!<%?$N$_J];}$5$l$k!#(B
   *
   * $BCm(B)$B8=:_$N<BAu$G$O!"0lHV:G8e$K3JG<$7$?%G!<%?$N$_%P%C%U%!$+$iFI$_=P$72DG=(B
   *
   * @param DataType $B%P%C%U%!$K3JG<$9$k%G!<%?7?(B
   *
   * @since 0.4.0
   *
   * @else
   * @class RingBuffer
   * @brief Ring buffer implementation class
   * 
   * This is the buffer implementation class with ring shaped buffer of
   * specified length.
   * If data is stored in the entire buffer, data from now on will be
   * overwritten from old data one by one.
   * Therefore, only the length of latest data is stored in the buffer.
   *
   * Note: In the current implementation, only last stored data can be read
   *       from the buffer.
   *
   * @param DataType Data type to store in the buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class RingBuffer
    : public BufferBase<DataType>
  {
  public:
    BUFFERSTATUS_ENUM
    typedef coil::Guard<coil::Mutex> Guard;
    /*!
     * @if jp
     *
     * @brief $B%3%s%9%H%i%/%?(B
     * 
     * $B%3%s%9%H%i%/%?(B
     * $B;XDj$5$l$?%P%C%U%!D9$G%P%C%U%!$r=i4|2=$9$k!#(B
     *
     * @param length $B%P%C%U%!D9(B
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor.
     * Initialize the buffer by specified buffer length.
     * However, if the specified length is less than two, the buffer should
     * be initialized by two in length.
     *
     * @param length Buffer length
     * 
     * @endif
     */
    RingBuffer(long int length = RINGBUFFER_DEFAULT_LENGTH)
      : m_overwrite(true), m_readback(true),
        m_timedwrite(false), m_timedread(false),
        m_wtimeout(1, 0), m_rtimeout(1, 0),
        m_length(length),
        m_wpos(0), m_rpos(0), m_fillcount(0), m_wcount(0),
        m_buffer(m_length)
    {
      this->reset();
    }
    
    /*!
     * @if jp
     *
     * @brief $B2>A[%G%9%H%i%/%?(B
     * 
     * $B2>A[%G%9%H%i%/%?!#(B
     * 
     * @else
     *
     * @brief Virtual destractor
     * 
     * Virtual destractor
     * 
     * @endif
     */
    virtual ~RingBuffer(void)
    {
    }
    
    /*!
     * @if jp
     * @brief $B%P%C%U%!$N@_Dj(B
     *
     * coil::Properties $B$GM?$($i$l$k%W%m%Q%F%#$K$h$j!"(B
     * $B%P%C%U%!$N@_Dj$r=i4|2=$9$k!#(B
     * $B;HMQ$G$-$k%*%W%7%g%s$H0UL#$O0J2<$NDL$j(B
     *
     * - buffer.length:
     *     $B%P%C%U%!$ND9$5!#<+A3?t0J30$N?tCM$,;XDj$5$l$F$bL5;k$5$l$k!#$9(B
     *     $B$G$K%P%C%U%!$,;HMQ>uBV$G$b!"D9$5$,:F@_Dj$5$l$?$N$A!"$9$Y$F$N(B
     *     $B%]%$%s%?$,=i4|2=$5$l$k!#(B
     *
     * - buffer.write.full_policy:
     *     $B>e=q$-$9$k$+$I$&$+$N%]%j%7!<!#(B
     *     overwrite ($B>e=q$-(B), do_nothing ($B2?$b$7$J$$(B), block ($B%V%m%C%/$9$k(B)
     *     block $B$r;XDj$7$?>l9g!"<!$N(B timeout $BCM$r;XDj$9$l$P!";XDj;~4V8e(B
     *     $B=q$-9~$_IT2DG=$G$"$l$P%?%$%`%"%&%H$9$k!#(B
     *     $B%G%U%)%k%H$O(B  overwrite ($B>e=q$-(B)$B!#(B
     *
     * - buffer.write.timeout:
     *     $B%?%$%`%"%&%H;~4V$r(B [sec] $B$G;XDj$9$k!#%G%U%)%k%H$O(B 1.0 [sec]$B!#(B
     *     1 sec -> 1.0, 1 ms -> 0.001, $B%?%$%`%"%&%H$7$J$$(B -> 0.0
     *
     * - buffer.read.empty_policy:
     *     $B%P%C%U%!$,6u$N$H$-$NFI$_=P$7%]%j%7!<!#(B
     *     readback ($B:G8e$NMWAG(B), do_nothing ($B2?$b$7$J$$(B), block ($B%V%m%C%/$9$k(B)
     *     block $B$r;XDj$7$?>l9g!"<!$N(B timeout $BCM$r;XDj$9$l$P!";XDj;~4V8e(B
     *     $BFI$_=P$7IT2DG=$G$"$l$P%?%$%`%"%&%H$9$k!#(B
     *     $B%G%U%)%k%H$O(B readback ($B:G8e$NMWAG(B)$B!#(B
     *
     * - buffer.read.timeout:
     *     $B%?%$%`%"%&%H;~4V(B [sec] $B$G;XDj$9$k!#%G%U%)%k%H$O(B 1.0 [sec]$B!#(B
     *     1sec -> 1.0, 1ms -> 0.001, $B%?%$%`%"%&%H$7$J$$(B -> 0.0
     *
     * @else
     *
     * @endif
     */
    virtual void init(const coil::Properties& prop)
    {
      initLength(prop);
      initWritePolicy(prop);
      initReadPolicy(prop);
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!D9$r<hF@$9$k(B
     * 
     * $B%P%C%U%!D9$r<hF@$9$k!#(B
     * 
     * @return $B%P%C%U%!D9(B
     * 
     * @else
     *
     * @brief Get the buffer length
     * 
     * Get the buffer length.
     * 
     * @return Buffer length
     * 
     *
     * @endif
     */
    virtual size_t length(void) const
    {
      Guard guard(m_posmutex);
      return m_length;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$ND9$5$r%;%C%H$9$k(B
     * 
     * $B%P%C%U%!D9$r@_Dj$9$k!#@_DjIT2D$J>l9g$O(BNOT_SUPPORTED$B$,JV$k!#(B
     * $B$3$N<BAu$G$O(B BUFFER_OK $B$7$+JV$5$J$$!#(B
     * 
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         NOT_SUPPORTED: $B%P%C%U%!D9JQ99IT2D(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */
    virtual ReturnCode length(size_t n)
    {
      m_buffer.resize(n);
      m_length = n;
      this->reset();
      return ::RTC::BufferStatus::BUFFER_OK; //BUFFER_OK;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$N>uBV$r%j%;%C%H$9$k(B
     * 
     * $B%P%C%U%!$NFI$_=P$7%]%$%s%?$H=q$-9~$_%]%$%s%?$N0LCV$r%j%;%C%H$9$k!#(B
     * $B$3$N<BAu$G$O(B BUFFER_OK $B$7$+JV$5$J$$!#(B
     * 
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         NOT_SUPPORTED: $B%j%;%C%HIT2DG=(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */ 
    virtual ReturnCode reset()
    {
      Guard guard(m_posmutex);
      m_fillcount = 0;
      m_wcount = 0;
      m_wpos = 0;
      m_rpos = 0;
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    
    
    //----------------------------------------------------------------------
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$N8=:_$N=q9~$_MWAG$N%]%$%s%?(B
     * 
     * $B%P%C%U%!$N8=:_$N=q9~$_MWAG$N%]%$%s%?$^$?$O!"(Bn$B8D@h$N%]%$%s%?$rJV$9(B
     * 
     * @param  n $B=q9~$_%]%$%s%?(B + n $B$N0LCV$N%]%$%s%?(B 
     * @return $B=q9~$_0LCV$N%]%$%s%?(B
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */ 
    virtual DataType* wptr(long int n = 0) 
    {
      Guard guard(m_posmutex);
      return &m_buffer[(m_wpos + n + m_length) % m_length];
    }
    
    /*!
     * @if jp
     *
     * @brief $B=q9~$_%]%$%s%?$r?J$a$k(B
     * 
     * $B8=:_$N=q$-9~$_0LCV$N%]%$%s%?$r(B n $B8D?J$a$k!#(B
     * $B=q$-9~$_2DG=$JMWAG?t0J>e$N?tCM$r;XDj$7$?>l9g!"(BPRECONDITION_NOT_MET
     * $B$rJV$9!#(B
     * 
     * @param  n $B=q9~$_%]%$%s%?(B + n $B$N0LCV$N%]%$%s%?(B 
     * @return BUFFER_OK:            $B@5>o=*N;(B
     *         PRECONDITION_NOT_MET: n > writable()
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */ 
    virtual ReturnCode advanceWptr(long int n = 1)
    {
      // n > 0 :
      //     n satisfies n <= writable elements
      //                 n <= m_length - m_fillcout
      // n < 0 : -n = n'
      //     n satisfies n'<= readable elements
      //                 n'<= m_fillcount
      //                 n >= - m_fillcount
      if ((n > 0 && n > static_cast<long int>(m_length - m_fillcount)) ||
          (n < 0 && n < static_cast<long int>(-m_fillcount)))
        {
          return ::RTC::BufferStatus::PRECONDITION_NOT_MET;
        }

      Guard guard(m_posmutex);
      m_wpos = (m_wpos + n + m_length) % m_length;
      m_fillcount += n;
      m_wcount += n;
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$K%G!<%?$r=q$-9~$`(B
     * 
     * $B%P%C%U%!$K%G!<%?$r=q$-9~$`!#=q$-9~$_%]%$%s%?$N0LCV$OJQ99$5$l$J$$!#(B
     * $B$3$N<BAu$G$O>o$K(B BUFFER_OK $B$rJV$9!#(B
     * 
     * @param value $B=q$-9~$_BP>]%G!<%?(B
     *
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     * Always BUFFER_OK will be returned in this implementation.
     *
     * @param value Target data to write.
     *
     * @return BUFFER_OK:    Successful
     *         BUFFER_ERROR: Failed
     *
     * @endif
     */
    virtual ReturnCode put(const DataType& value)
    {
      Guard guard(m_posmutex);
      m_buffer[m_wpos] = value;
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$K=q$-9~$`(B
     * 
     * $B0z?t$GM?$($i$l$?%G!<%?$r%P%C%U%!$K=q$-9~$`!#(B
     *
     * $BBh(B2$B0z?t(B(sec)$B!"Bh(B3$B0z?t(B(nsec)$B$,;XDj$5$l$F$$$J$$>l9g!"%P%C%U%!%U%k(B
     * $B;~$N=q9~$_%b!<%I(B (overwrite, do_nothing, block) $B$O(B init() $B$G@_Dj(B
     * $B$5$l$?%b!<%I$K=>$&!#(B
     *
     * $BBh(B2$B0z?t(B(sec) $B$K0z?t$,;XDj$5$l$?>l9g$O!"(Binit() $B$G@_Dj$5$l$?%b!<%I(B
     * $B$K4X$o$i$:!"(Bblock $B%b!<%I$H$J$j!"%P%C%U%!$,%U%k>uBV$G$"$l$P;XDj;~(B
     * $B4V$^$A!"%?%$%`%"%&%H$9$k!#Bh(B3$B0z?t(B(nsec)$B$O;XDj$5$l$J$$>l9g(B0$B$H$7$F(B
     * $B07$o$l$k!#%?%$%`%"%&%HBT$ACf$K!"FI$_=P$7%9%l%C%IB&$G%P%C%U%!$+$i(B
     * $BFI$_=P$;$P!"%V%m%C%-%s%0$O2r=|$5$l%G!<%?$,=q$-9~$^$l$k!#(B
     *
     * $B=q$-9~$_;~$K%P%C%U%!$,6u(B(empty)$B>uBV$G!"JL$N%9%l%C%I$,(Bblock$B%b!<%I(B
     * $B$GFI$_=P$7BT$A$r$7$F$$$k>l9g!"(Bsignal$B$rH/9T$7$FFI$_=P$7B&$N%V%m%C(B
     * $B%-%s%0$,2r=|$5$l$k!#(B
     * 
     * @param value $B=q$-9~$_BP>]%G!<%?(B
     * @param sec   $B%?%$%`%"%&%H;~4V(B sec  (default -1: $BL58z(B)
     * @param nsec  $B%?%$%`%"%&%H;~4V(B nsec (default 0)
     * @return BUFFER_OK            $B@5>o=*N;(B
     *         BUFFER_FULL          $B%P%C%U%!$,%U%k>uBV(B
     *         TIMEOUT              $B=q9~$_$,%?%$%`%"%&%H$7$?(B
     *         PRECONDITION_NOT_MET $B@_Dj0[>o(B
     * 
     * @else
     *
     * @brief Write data into the buffer
     * 
     * Write data which is given argument into the buffer.
     *
     * @param value Target data for writing
     *
     * @return Writing result (Always true: writing success is returned)
     * 
     * @endif
     */
    virtual ReturnCode write(const DataType& value,
                             long int sec = -1, long int nsec = 0)
    {
      {
      Guard guard(m_full.mutex);
        
      if (full())
        {
          
          bool timedwrite(m_timedwrite);
          bool overwrite(m_overwrite);

          if (!(sec < 0)) // if second arg is set -> block mode
            {
              timedwrite = true;
              overwrite  = false;
            }

          if (overwrite && !timedwrite)       // "overwrite" mode
            {
              advanceRptr();
            }
          else if (!overwrite && !timedwrite) // "do_nothing" mode
            {
              return ::RTC::BufferStatus::BUFFER_FULL;
            }
          else if (!overwrite && timedwrite)  // "block" mode
            {
              if (sec < 0)
                {
                  sec = m_wtimeout.sec();
                  nsec = m_wtimeout.usec() * 1000;
                }
              //  true: signaled, false: timeout
              if (!m_full.cond.wait(sec, nsec))
                {
                  return ::RTC::BufferStatus::TIMEOUT;
                }
            }
          else                                    // unknown condition
            {
              return ::RTC::BufferStatus::PRECONDITION_NOT_MET;
            }
        }
      }      
    
      put(value);
      
      if (empty())
        {
          Guard eguard(m_empty.mutex);
          advanceWptr(1);
          m_empty.cond.signal();
        }
      else
        {
          advanceWptr(1);
        }
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$K=q9~$_2DG=$JMWAG?t(B
     * 
     * $B%P%C%U%!$K=q9~$_2DG=$JMWAG?t$rJV$9!#(B
     * 
     * @return $B=q$-9~$_2DG=$JMWAG?t(B
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual size_t writable() const
    {
      Guard guard(m_posmutex);
      return m_length - m_fillcount;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!(Bfull$B%A%'%C%/(B
     * 
     * $B%P%C%U%!(Bfull$B%A%'%C%/MQ=c?h2>A[4X?t(B
     *
     * @return full$B%A%'%C%/7k2L(B(true:$B%P%C%U%!(Bfull$B!$(Bfalse:$B%P%C%U%!6u$-$"$j(B)
     * 
     * @else
     *
     * @brief Check on whether the buffer is full.
     *
     * Pure virtual function to check on whether the buffer is full.
     *
     * @return True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool full(void) const
    {
      Guard guard(m_posmutex);
      return m_length == m_fillcount;
    }
    
    //----------------------------------------------------------------------
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$N8=:_$NFI$_=P$7MWAG$N%]%$%s%?(B
     * 
     * $B%P%C%U%!$N8=:_$NFI$_=P$7MWAG$N%]%$%s%?$^$?$O!"(Bn$B8D@h$N%]%$%s%?$rJV$9(B
     * 
     * @param  n $BFI$_=P$7%]%$%s%?(B + n $B$N0LCV$N%]%$%s%?(B 
     * @return $BFI$_=P$70LCV$N%]%$%s%?(B
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */ 
    virtual DataType* rptr(long int n = 0)
    {
      Guard guard(m_posmutex);
      return &(m_buffer[(m_rpos + n + m_length) % m_length]);
    }
    
    /*!
     * @if jp
     *
     * @brief $BFI$_=P$7%]%$%s%?$r?J$a$k(B
     * 
     * $B8=:_$NFI$_=P$70LCV$N%]%$%s%?$r(B n $B8D?J$a$k!#(B
     * 
     * @param  n $BFI$_=P$7%]%$%s%?(B + n $B$N0LCV$N%]%$%s%?(B 
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */ 
    virtual ReturnCode advanceRptr(long int n = 1)
    {
      // n > 0 :
      //     n satisfies n <= readable elements
      //                 n <= m_fillcout 
      // n < 0 : -n = n'
      //     n satisfies n'<= m_length - m_fillcount
      //                 n >= m_fillcount - m_length
      if ((n > 0 && n > static_cast<long int>(m_fillcount)) ||
          (n < 0 && n < static_cast<long int>(m_fillcount - m_length)))
        {
          return ::RTC::BufferStatus::PRECONDITION_NOT_MET;
        }

      Guard guard(m_posmutex);
      m_rpos = (m_rpos + n + m_length) % m_length;
      m_fillcount -= n;
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$+$i%G!<%?$rFI$_=P$9(B
     * 
     * $B%P%C%U%!$+$i%G!<%?$rFI$_$@$9!#FI$_=P$7%]%$%s%?$N0LCV$OJQ99$5$l$J$$!#(B
     * 
     * @param value $BFI$_=P$7%G!<%?(B
     *
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual ReturnCode get(DataType& value)
    {
      Guard gaurd(m_posmutex);
      value = m_buffer[m_rpos];
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$+$i%G!<%?$rFI$_=P$9(B
     * 
     * $B%P%C%U%!$+$i%G!<%?$rFI$_$@$9!#FI$_=P$7%]%$%s%?$N0LCV$OJQ99$5$l$J$$!#(B
     *
     * @return $BFI$_=P$7%G!<%?(B
     * 
     * @else
     *
     * @brief Reading data from the buffer
     *
     * @return Read data
     *
     * @endif
     */
    virtual DataType& get()
    {
      Guard gaurd(m_posmutex);
      return m_buffer[m_rpos];
    }
    
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$+$iFI$_=P$9(B
     * 
     * $B%P%C%U%!$K3JG<$5$l$?%G!<%?$rFI$_=P$9!#(B
     *
     * $BBh(B2$B0z?t(B(sec)$B!"Bh(B3$B0z?t(B(nsec)$B$,;XDj$5$l$F$$$J$$>l9g!"%P%C%U%!6u>u(B
     * $BBV$G$NFI$_=P$7%b!<%I(B (readback, do_nothing, block) $B$O(B init() $B$G@_(B
     * $BDj$5$l$?%b!<%I$K=>$&!#(B
     *
     * $BBh(B2$B0z?t(B(sec) $B$K0z?t$,;XDj$5$l$?>l9g$O!"(Binit() $B$G@_Dj$5$l$?%b!<%I(B
     * $B$K4X$o$i$:!"(Bblock $B%b!<%I$H$J$j!"%P%C%U%!$,6u>uBV$G$"$l$P;XDj;~4V(B
     * $BBT$A!"%?%$%`%"%&%H$9$k!#Bh(B3$B0z?t(B(nsec)$B$O;XDj$5$l$J$$>l9g(B0$B$H$7$F07(B
     * $B$o$l$k!#%?%$%`%"%&%HBT$ACf$K!"=q9~$_%9%l%C%IB&$G%P%C%U%!$X=q9~$_(B
     * $B$,$"$l$P!"%V%m%C%-%s%0$O2r=|$5$l%G!<%?$,FI$_$@$5$l$k!#(B
     *
     * $BFI$_=P$7;~$K%P%C%U%!$,6u(B(empty)$B>uBV$G!"JL$N%9%l%C%I$,(Bblock$B%b!<%I(B
     * $B$G=q9~$_BT$A$r$7$F$$$k>l9g!"(Bsignal$B$rH/9T$7$F=q9~$_B&$N%V%m%C%-%s(B
     * $B%0$,2r=|$5$l$k!#(B
     * 
     * @param value $BFI$_=P$7BP>]%G!<%?(B
     * @param sec   $B%?%$%`%"%&%H;~4V(B sec  (default -1: $BL58z(B)
     * @param nsec  $B%?%$%`%"%&%H;~4V(B nsec (default 0)
     * @return BUFFER_OK            $B@5>o=*N;(B
     *         BUFFER_EMPTY         $B%P%C%U%!$,6u>uBV(B
     *         TIMEOUT              $B=q9~$_$,%?%$%`%"%&%H$7$?(B
     *         PRECONDITION_NOT_MET $B@_Dj0[>o(B
     * 
     * @else
     *
     * @brief Readout data from the buffer
     * 
     * Readout data stored into the buffer.
     * 
     * @param value Readout data
     *
     * @return Readout result (Always true: readout success is returned)
     * 
     * @endif
     */
    virtual ReturnCode read(DataType& value,
                            long int sec = -1, long int nsec = 0)
    {
      {
      Guard gaurd(m_empty.mutex);
      
      if (empty())
        {
          bool timedread(m_timedread);
          bool readback(m_readback);

          if (!(sec < 0)) // if second arg is set -> block mode
            {
              timedread = true;
              readback  = false;
              sec = m_rtimeout.sec();
              nsec = m_rtimeout.usec() * 1000;
            }

          if (readback && !timedread)       // "readback" mode
            {
              if (!(m_wcount > 0))
                {
                  return ::RTC::BufferStatus::BUFFER_EMPTY;
                }
              advanceRptr(-1);
            }
          else if (!readback && !timedread) // "do_nothing" mode
            {
              return ::RTC::BufferStatus::BUFFER_EMPTY;
            }
          else if (!readback && timedread)  // "block" mode
            {
              if (sec < 0)
                {
                  sec = m_rtimeout.sec();
                  nsec = m_rtimeout.usec() * 1000;
                }
              //  true: signaled, false: timeout
              if (!m_empty.cond.wait(sec, nsec))
                {
                  return ::RTC::BufferStatus::TIMEOUT;
                }
            }
          else                                    // unknown condition
            {
              return ::RTC::BufferStatus::PRECONDITION_NOT_MET;
            }
        }
      }
      
      get(value);

      if (full())
        {
          Guard fguard(m_full.mutex);
          advanceRptr(1);
          m_full.cond.signal();
        }
      else
        {
          advanceRptr(1);
        }
      return ::RTC::BufferStatus::BUFFER_OK;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!$+$iFI$_=P$72DG=$JMWAG?t(B
     * 
     * $B%P%C%U%!$+$iFI$_=P$72DG=$JMWAG?t$rJV$9!#(B
     * 
     * @return $BFI$_=P$72DG=$JMWAG?t(B
     *
     * @return BUFFER_OK: $B@5>o=*N;(B
     *         BUFFER_ERROR: $B0[>o=*N;(B
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual size_t readable() const
    {
      Guard guard(m_posmutex);
      return m_fillcount;
    }
    
    /*!
     * @if jp
     *
     * @brief $B%P%C%U%!(Bempty$B%A%'%C%/(B
     * 
     * $B%P%C%U%!(Bempty$B%A%'%C%/MQ=c?h2>A[4X?t(B
     *
     * @return empty$B%A%'%C%/7k2L(B(true:$B%P%C%U%!(Bempty$B!$(Bfalse:$B%P%C%U%!%G!<%?$"$j(B)
     * 
     * @else
     *
     * @brief Check on whether the buffer is empty.
     *
     * Pure virtual function to check on whether the buffer is empty.
     *
     * @return True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool empty(void) const
    {
      Guard guard(m_posmutex);
      return m_fillcount == 0;
    }
    
  private:
    inline void initLength(const coil::Properties& prop)
    {
      if (!prop["length"].empty())
        {
          size_t n;
          if (coil::stringTo(n, prop["length"].c_str()))
            {
              if (n > 0)
                {
                  this->length(n);
                }
            }
        }
    }
    
    inline void initWritePolicy(const coil::Properties& prop)
    {
      std::string policy(prop["write.full_policy"]);
      coil::normalize(policy);
      if (policy == "overwrite")
        {
          m_overwrite = true;
          m_timedwrite = false;
        }
      else if (policy == "do_nothing")
        {
          m_overwrite = false;
          m_timedwrite = false;
        }
      else if (policy == "block")
        {
          m_overwrite = false;
          m_timedwrite = true;
          
          double tm;
          if (coil::stringTo(tm, prop["write.timeout"].c_str()))
            {
              if (!(tm < 0))
                {
                  m_wtimeout = tm;
                }
            }
        }
    }
    
    inline void initReadPolicy(const coil::Properties& prop)
    {
      std::string policy(prop["read.empty_policy"]);
      if (policy == "readback")
        {
          m_readback = true;
          m_timedread = false;
        }
      else if (policy == "do_nothing")
        {
          m_readback = false;
          m_timedread = false;
        }
      else if (policy == "block")
        {
          m_readback = false;
          m_timedread = true;
          double tm;
          if (coil::stringTo(tm, prop["read.timeout"].c_str()))
            {
              m_rtimeout = tm;
            }
        }
    }
    
  private:
    /*!
     * @if jp
     * @brief $B>e=q$-%U%i%0(B
     * @else
     * @brief Overwrite flag
     * @endif
     */
    bool m_overwrite;

    /*!
     * @if jp
     * @brief $BFI$_La$7%U%i%0(B
     * @else
     * @brief Readback flag
     * @endif
     */
    bool m_readback;

    /*!
     * @if jp
     * @brief $B%?%$%`%"%&%HIU$-=q$-9~$_%U%i%0(B
     * @else
     * @brief Timedwrite flag
     * @endif
     */
    bool m_timedwrite;
    /*!
     * @if jp
     * @brief $B%?%$%`%"%&%HIU$-FI$_=P$7%U%i%0(B
     * @else
     * @brief Timedread flag
     * @endif
     */
    bool m_timedread;

    /*!
     * @if jp
     * @brief $B=q$-9~$_;~%?%$%`%"%&%H(B
     * @else
     * @brief Timeout time for writing
     * @endif
     */
    coil::TimeValue m_wtimeout;

    /*!
     * @if jp
     * @brief $BFI$_=P$7;~%?%$%`%"%&%H(B
     * @else
     * @brief Timeout time of reading
     * @endif
     */
    coil::TimeValue m_rtimeout;

    /*!
     * @if jp
     * @brief $B%P%C%U%!D9(B
     * @else
     * @brief Buffer length
     * @endif
     */
    size_t m_length;

    /*!
     * @if jp
     * @brief $B=q$-9~$_%]%$%s%?(B
     * @else
     * @brief pointer to write
     * @endif
     */
    size_t m_wpos;

    /*!
     * @if jp
     * @brief $BFI$_=P$7%]%$%s%?(B
     * @else
     * @brief poitner to read
     * @endif
     */
    size_t m_rpos;

    /*!
     * @if jp
     * @brief Fill$B%+%&%s%H(B
     * @else
     * @brief Fill count
     * @endif
     */
    size_t m_fillcount;

    /*!
     * @if jp
     * @brief $B=q$-9~$_%+%&%s%H(B
     * @else
     * @brief Counter for writing
     * @endif
     */
    size_t m_wcount;

    /*!
     * @if jp
     * @brief $B%P%C%U%!G[Ns(B
     * @else
     * @brief baffer array
     * @endif
     */
    std::vector<DataType> m_buffer;
    
    /*!
     * @if jp
     * @brief $B>r7oJQ?t9=B$BN(B
     * @else
     * @brief struct for condition variable
     * @endif
     */
    struct condition
    {
      condition() : cond(mutex) {}
      coil::Condition<coil::Mutex> cond;
      coil::Mutex mutex;
    };
    
    /*!
     * @if jp
     * @brief $B0LCVJQ?t%_%e!<%F%C%/%9(B
     * @else
     * @brief mutex for position variable
     * @endif
     */
    mutable coil::Mutex m_posmutex;

    /*!
     * @if jp
     * @brief $B6u>r7oJQ?t(B
     * @else
     * @brief empty condition variable
     * @endif
     */
    condition m_empty;

    /*!
     * @if jp
     * @brief $BK~GU>r7oJQ?t(B
     * @else
     * @brief full condition variable
     * @endif
     */
    condition m_full;
  };
}; // namespace RTC

#endif // RTC_RINGBUFFER_H
